@@ -1,6 +1,6 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * Copyright (C) 2001-2016, Daniel Roux, EPSITEC SA & TerranovaTeam
  * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
@@ -34,6 +34,7 @@
 #include "common/logger.h"
 #include "common/make_unique.h"
 #include "common/restext.h"
+#include "common/version.h"
 
 #include "common/resources/resourcemanager.h"
 
@@ -43,6 +44,7 @@
 
 #include <memory>
 #include <vector>
+#include <boost/filesystem.hpp>
 
 /* Doxygen main page */
 
@@ -88,9 +90,28 @@ The current layout is the following:
 extern "C"
 {
 
-int SDL_MAIN_FUNC(int argc, char *argv[])
+int main(int argc, char *argv[])
 {
     CLogger logger; // single instance of logger
+    logger.AddOutput(stderr);
+
+    auto systemUtils = CSystemUtils::Create(); // platform-specific utils
+    systemUtils->Init();
+
+    // Add file output to the logger
+    std::string logFileName;
+    #if DEV_BUILD
+        logFileName = "log.txt";
+    #else
+        boost::filesystem::create_directories(systemUtils->GetSaveDir());
+        logFileName = systemUtils->GetSaveDir() + "/log.txt";
+    #endif
+    FILE* logFile = fopen(logFileName.c_str(), "w");
+    if (logFile)
+        logger.AddOutput(logFile);
+    else
+        logger.Error("Failed to create log file, writing log to file disabled\n");
+
 
     // Workaround for character encoding in argv on Windows
     #if PLATFORM_WINDOWS
@@ -123,9 +144,6 @@ int SDL_MAIN_FUNC(int argc, char *argv[])
 
     logger.Info("%s starting\n", COLOBOT_FULLNAME);
 
-    auto systemUtils = CSystemUtils::Create(); // platform-specific utils
-    systemUtils->Init();
-
     CSignalHandlers::Init(systemUtils.get());
 
     CResourceManager manager(argv[0]);
