@@ -1,6 +1,6 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * Copyright (C) 2001-2016, Daniel Roux, EPSITEC SA & TerranovaTeam
  * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
@@ -62,20 +62,29 @@ enum EventType
     EVENT_MOUSE_WHEEL       = 5,
     //! Event sent after moving the mouse
     EVENT_MOUSE_MOVE        = 7,
-    //! Event sent after pressing a key
-    EVENT_KEY_DOWN          = 8,
-    //! Event sent after releasing a key
-    EVENT_KEY_UP            = 9,
+    //! Event sent when mouse enters the window
+    EVENT_MOUSE_ENTER       = 8,
+    //! Event sent when mouse leaves the window
+    EVENT_MOUSE_LEAVE       = 9,
 
-    //! Event sent when application window loses/gains focus
-    EVENT_ACTIVE            = 10,
+    //! Event sent after pressing a key
+    EVENT_KEY_DOWN          = 10,
+    //! Event sent after releasing a key
+    EVENT_KEY_UP            = 11,
+    //! Event sent when user inputs some character
+    EVENT_TEXT_INPUT        = 12,
 
     //! Event sent after moving joystick axes
-    EVENT_JOY_AXIS          = 12,
+    EVENT_JOY_AXIS          = 13,
     //! Event sent after pressing a joystick button
-    EVENT_JOY_BUTTON_DOWN   = 13,
+    EVENT_JOY_BUTTON_DOWN   = 14,
     //! Event sent after releasing a joystick button
-    EVENT_JOY_BUTTON_UP     = 14,
+    EVENT_JOY_BUTTON_UP     = 15,
+
+    //! Event sent when the app winddow gains focus
+    EVENT_FOCUS_GAINED      = 16,
+    //! Event sent when the app winddow loses focus
+    EVENT_FOCUS_LOST        = 17,
 
     //!< Maximum value of system events
     EVENT_SYS_MAX,
@@ -86,6 +95,10 @@ enum EventType
     //! Event sent on user quit request
     EVENT_QUIT              = 20,
     EVENT_UPDINTERFACE      = 21,
+    //! Event sent on resolution change
+    EVENT_RESOLUTION_CHANGED = 22,
+    //! Event sent when textures have to be reloaded
+    EVENT_RELOAD_TEXTURES   = 23,
     EVENT_WIN               = 30,
     EVENT_LOST              = 31,
 
@@ -136,7 +149,7 @@ enum EventType
     EVENT_WINDOW4           = 84,   //!< CDisplayInfo
     EVENT_WINDOW5           = 85,   //!< all menu windows
     EVENT_WINDOW6           = 86,   //!< code battle interface
-    EVENT_WINDOW7           = 87,   //!< (unused)
+    EVENT_WINDOW7           = 87,   //!< debug interface
     EVENT_WINDOW8           = 88,   //!< (unused)
     EVENT_WINDOW9           = 89,   //!< CMainDialog and CStudio file selector
 
@@ -201,6 +214,7 @@ enum EventType
     EVENT_INTERFACE_READ    = 411,
     EVENT_INTERFACE_ABORT   = 412,
     EVENT_INTERFACE_USER    = 413,
+    EVENT_INTERFACE_SATCOM  = 414,
 
     EVENT_INTERFACE_CHAP    = 420,
     EVENT_INTERFACE_LIST    = 421,
@@ -223,6 +237,7 @@ enum EventType
     EVENT_INTERFACE_LIGHT   = 457,
     EVENT_INTERFACE_PARTI   = 458,
     EVENT_INTERFACE_CLIP    = 459,
+    EVENT_INTERFACE_PAUSE_BLUR = 460,
     EVENT_INTERFACE_RAIN    = 462,
     EVENT_INTERFACE_GLINT   = 463,
     EVENT_INTERFACE_TOOLTIP = 464,
@@ -231,7 +246,7 @@ enum EventType
     EVENT_INTERFACE_INVERTX = 468,
     EVENT_INTERFACE_INVERTY = 469,
     EVENT_INTERFACE_EFFECT  = 470,
-    EVENT_INTERFACE_MOUSE   = 471,
+    EVENT_INTERFACE_BGPAUSE = 471,
     EVENT_INTERFACE_FOG     = 474,
     EVENT_INTERFACE_EDITMODE= 476,
     EVENT_INTERFACE_EDITVALUE= 477,
@@ -247,6 +262,7 @@ enum EventType
     EVENT_INTERFACE_SHADOW_MAPPING = 787,
     EVENT_INTERFACE_SHADOW_MAPPING_QUALITY = 788,
     EVENT_INTERFACE_SHADOW_MAPPING_BUFFER = 789,
+    EVENT_INTERFACE_LANGUAGE = 790,
 
     EVENT_INTERFACE_KINFO1  = 500,
     EVENT_INTERFACE_KINFO2  = 501,
@@ -269,17 +285,30 @@ enum EventType
 
     EVENT_INTERFACE_JOYSTICK= 560,
     EVENT_INTERFACE_SOLUCE  = 561,
+    EVENT_INTERFACE_JOYSTICK_DEADZONE = 562,
+    EVENT_INTERFACE_JOYSTICK_X = 563,
+    EVENT_INTERFACE_JOYSTICK_Y = 564,
+    EVENT_INTERFACE_JOYSTICK_Z = 565,
+    EVENT_INTERFACE_JOYSTICK_CAM_X = 566,
+    EVENT_INTERFACE_JOYSTICK_CAM_Y = 567,
+    EVENT_INTERFACE_JOYSTICK_CAM_Z = 568,
+    EVENT_INTERFACE_JOYSTICK_X_INVERT = 569,
+    EVENT_INTERFACE_JOYSTICK_Y_INVERT = 570,
+    EVENT_INTERFACE_JOYSTICK_Z_INVERT = 571,
+    EVENT_INTERFACE_JOYSTICK_CAM_X_INVERT = 572,
+    EVENT_INTERFACE_JOYSTICK_CAM_Y_INVERT = 573,
+    EVENT_INTERFACE_JOYSTICK_CAM_Z_INVERT = 574,
 
-    EVENT_INTERFACE_GLINTl  = 570,
-    EVENT_INTERFACE_GLINTr  = 571,
-    EVENT_INTERFACE_GLINTu  = 572,
-    EVENT_INTERFACE_GLINTb  = 573,
+    EVENT_INTERFACE_GLINTl  = 590,
+    EVENT_INTERFACE_GLINTr  = 591,
+    EVENT_INTERFACE_GLINTu  = 592,
+    EVENT_INTERFACE_GLINTb  = 593,
 
-    EVENT_INTERFACE_NEDIT   = 580,
-    EVENT_INTERFACE_NLIST   = 581,
-    EVENT_INTERFACE_NOK     = 582,
-    EVENT_INTERFACE_NDELETE = 584,
-    EVENT_INTERFACE_NLABEL  = 585,
+    EVENT_INTERFACE_NEDIT   = 595,
+    EVENT_INTERFACE_NLIST   = 596,
+    EVENT_INTERFACE_NOK     = 597,
+    EVENT_INTERFACE_NDELETE = 598,
+    EVENT_INTERFACE_NLABEL  = 599,
 
     EVENT_INTERFACE_IOWRITE = 600,
     EVENT_INTERFACE_IOREAD  = 601,
@@ -358,6 +387,31 @@ enum EventType
     EVENT_CMD               = 800,
     EVENT_SPEED             = 801,
 
+    EVENT_DBG_STATS         = 850,
+    EVENT_DBG_SPAWN_OBJ     = 851,
+    EVENT_DBG_TELEPORT      = 852,
+    EVENT_DBG_LIGHTNING     = 853,
+    EVENT_DBG_RESOURCES     = 854,
+    EVENT_DBG_GOTO          = 855,
+    EVENT_DBG_CRASHSPHERES  = 856,
+    EVENT_DBG_LIGHTS        = 857,
+    EVENT_DBG_LIGHTS_DUMP   = 858,
+
+    EVENT_SPAWN_CANCEL      = 860,
+    EVENT_SPAWN_ME          = 861,
+    EVENT_SPAWN_WHEELEDGRABBER = 862,
+    EVENT_SPAWN_WHEELEDSHOOTER = 863,
+    EVENT_SPAWN_PHAZERSHOOTER  = 864,
+    EVENT_SPAWN_BOTFACTORY  = 865,
+    EVENT_SPAWN_CONVERTER   = 866,
+    EVENT_SPAWN_DERRICK     = 867,
+    EVENT_SPAWN_POWERSTATION= 868,
+    EVENT_SPAWN_TITANIUM    = 869,
+    EVENT_SPAWN_TITANIUMORE = 870,
+    EVENT_SPAWN_URANIUMORE  = 871,
+    EVENT_SPAWN_POWERCELL   = 872,
+    EVENT_SPAWN_NUCLEARCELL = 873,
+
     EVENT_HYPER_PREV        = 900,
     EVENT_HYPER_NEXT        = 901,
     EVENT_HYPER_HOME        = 902,
@@ -569,13 +623,25 @@ struct KeyEventData : public EventData
     bool virt = false;
     //! Key symbol: KEY(...) macro value or virtual key VIRTUAL_... (from common/key.h)
     unsigned int key = 0;
-    //! Unicode character
-    //! NOTE: applicable only to EVENT_KEY_DOWN events!
-    unsigned int unicode = 0;
     //! Input binding slot for this key
-    InputSlot slot = INPUT_SLOT_LEFT;
+    InputSlot slot = INPUT_SLOT_MAX;
 };
 
+/**
+ * \struct TextInputData
+ * \brief Additional data for text input event
+ */
+ struct TextInputData : public EventData
+ {
+    std::unique_ptr<EventData> Clone() const override
+    {
+        return MakeUnique<TextInputData>(*this);
+    }
+
+    //! Text entered by the user (usually one character, UTF-8 encoded)
+    std::string text = "";
+ };
+
 /**
  * \enum MouseButton
  * \brief Mouse button
@@ -606,16 +672,6 @@ struct MouseButtonEventData : public EventData
     MouseButton button = MOUSE_BUTTON_LEFT;
 };
 
-/**
- * \enum WheelDirection
- * \brief Direction of mouse wheel movement
- */
-enum WheelDirection
-{
-    WHEEL_UP,
-    WHEEL_DOWN
-};
-
 /**
  * \struct MouseWheelEventData
  * \brief Additional data for mouse wheel event.
@@ -627,8 +683,10 @@ struct MouseWheelEventData : public EventData
         return MakeUnique<MouseWheelEventData>(*this);
     }
 
-    //! Wheel direction
-    WheelDirection dir = WHEEL_UP;
+    //! Amount scrolled vertically, positive value is away from the user
+    signed int y = 0;
+    //! Amount scrolled horizontally (if the mouse supports it), positive value is to the right
+    signed int x = 0;
 };
 
 /**
@@ -663,38 +721,6 @@ struct JoyButtonEventData : public EventData
     unsigned char button = 0;
 };
 
-/**
- * \enum ActiveEventFlags
- * \brief Type of focus gained/lost
- */
-enum ActiveEventFlags
-{
-    //! Application window focus
-    ACTIVE_APP   = 0x01,
-    //! Input focus
-    ACTIVE_INPUT = 0x02,
-    //! Mouse focus
-    ACTIVE_MOUSE = 0x04
-
-};
-
-/**
- * \struct ActiveEventData
- * \brief Additional data for active event
- */
-struct ActiveEventData : public EventData
-{
-    std::unique_ptr<EventData> Clone() const override
-    {
-        return MakeUnique<ActiveEventData>(*this);
-    }
-
-    //! Flags (bitmask of enum values ActiveEventFlags)
-    unsigned char flags = 0;
-    //! True if the focus was gained; false otherwise
-    bool gain = false;
-};
-
 /**
  * \struct Event
  * \brief Event sent by system, interface or game
