@@ -1,6 +1,6 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * Copyright (C) 2001-2016, Daniel Roux, EPSITEC SA & TerranovaTeam
  * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
@@ -69,7 +69,7 @@ public:
     #endif
 
 private:
-    CSingleton& operator=(const CSingleton<T> &);
-    CSingleton(const CSingleton<T> &);
+    CSingleton& operator=(const CSingleton<T> &) = delete;
+    CSingleton(const CSingleton<T> &) = delete;
 };
 
