@@ -1,6 +1,6 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * Copyright (C) 2001-2016, Daniel Roux, EPSITEC SA & TerranovaTeam
  * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
@@ -27,9 +27,6 @@ namespace Gfx
 
 CNullDevice::CNullDevice()
 {
-    m_matrix = Math::Matrix();
-    m_material = Material();
-    m_light = Light();
 }
 
 CNullDevice::~CNullDevice()
@@ -44,6 +41,11 @@ void CNullDevice::DebugLights()
 {
 }
 
+std::string CNullDevice::GetName()
+{
+    return std::string("Null Device");
+}
+
 bool CNullDevice::Create()
 {
     return true;
@@ -69,6 +71,10 @@ void CNullDevice::Clear()
 {
 }
 
+void CNullDevice::SetRenderMode(RenderMode mode)
+{
+}
+
 void CNullDevice::SetTransform(TransformType type, const Math::Matrix &matrix)
 {
 }
@@ -111,6 +117,10 @@ Texture CNullDevice::CreateDepthTexture(int width, int height, int depth)
     return tex;
 }
 
+void CNullDevice::UpdateTexture(const Texture& texture, Math::IntPoint offset, ImageData* data, TexImgFormat format)
+{
+}
+
 void CNullDevice::DestroyTexture(const Texture &texture)
 {
 }
@@ -144,10 +154,6 @@ void CNullDevice::SetTextureStageWrap(int index, TexWrapMode wrapS, TexWrapMode
 {
 }
 
-void CNullDevice::SetTextureCoordGeneration(int index, TextureGenerationParams &params)
-{
-}
-
 void CNullDevice::DrawPrimitive(PrimitiveType type, const Vertex *vertices, int vertexCount,
                               Color color)
 {
@@ -162,6 +168,21 @@ void CNullDevice::DrawPrimitive(PrimitiveType type, const VertexCol *vertices, i
 {
 }
 
+void CNullDevice::DrawPrimitives(PrimitiveType type, const Vertex *vertices,
+    int first[], int count[], int drawCount, Color color)
+{
+}
+
+void CNullDevice::DrawPrimitives(PrimitiveType type, const VertexTex2 *vertices,
+    int first[], int count[], int drawCount, Color color)
+{
+}
+
+void CNullDevice::DrawPrimitives(PrimitiveType type, const VertexCol *vertices,
+    int first[], int count[], int drawCount)
+{
+}
+
 unsigned int CNullDevice::CreateStaticBuffer(PrimitiveType primitiveType, const Vertex* vertices, int vertexCount)
 {
     return 0;
