@@ -1,6 +1,6 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * Copyright (C) 2001-2016, Daniel Roux, EPSITEC SA & TerranovaTeam
  * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
@@ -34,6 +34,7 @@
 #include "math/intpoint.h"
 #include "math/matrix.h"
 #include "math/point.h"
+#include "math/sphere.h"
 #include "math/vector.h"
 
 
@@ -46,10 +47,8 @@
 
 
 class CApplication;
-class CObject;
 class CSoundInterface;
 class CImage;
-class CPauseManager;
 class CSystemUtils;
 struct SystemTimeStamp;
 struct Event;
@@ -483,40 +482,6 @@ enum EngineMouseType
     ENG_MOUSE_COUNT
 };
 
-/**
- * \struct EngineMouse
- * \brief Information about mouse cursor
- */
-struct EngineMouse
-{
-    //! Index of texture element for 1st image
-    int icon1;
-    //! Index of texture element for 2nd image
-    int icon2;
-    //! Shadow texture part
-    int iconShadow;
-    //! Mode to render 1st image in
-    EngineRenderState mode1;
-    //! Mode to render 2nd image in
-    EngineRenderState mode2;
-    //! Hot point
-    Math::Point hotPoint;
-
-    EngineMouse(int icon1 = -1,
-                int icon2 = -1,
-                int iconShadow = -1,
-                EngineRenderState mode1 = ENG_RSTATE_NORMAL,
-                EngineRenderState mode2 = ENG_RSTATE_NORMAL,
-                Math::Point hotPoint = Math::Point())
-     : icon1(icon1)
-     , icon2(icon2)
-     , iconShadow(iconShadow)
-     , mode1(mode1)
-     , mode2(mode2)
-     , hotPoint(hotPoint)
-    {}
-};
-
 
 /**
  * \class CEngine
@@ -646,8 +611,6 @@ public:
     CPlanet*        GetPlanet();
     //! Returns the fog manager
     CCloud*         GetCloud();
-    //! Returns the pause manager
-    CPauseManager*  GetPauseManager();
 
     //! Sets the terrain object
     void            SetTerrain(CTerrain* terrain);
@@ -658,9 +621,6 @@ public:
     //! Frees all resources before exit
     void            Destroy();
 
-    //! Resets some states and flushes textures after device was changed (e.g. resoulution changed)
-    void            ResetAfterDeviceChanged();
-
 
     //! Called once per frame, the call is the entry point for rendering
     void            Render();
@@ -677,8 +637,11 @@ public:
     void            WriteScreenShot(const std::string& fileName);
 
 
-    //! Get pause mode
-    TEST_VIRTUAL bool GetPause();
+    //@{
+    //! Management of animation pause mode
+    void            SetPause(bool pause);
+    bool            GetPause();
+    //@}
 
     //@{
     //! Management of displaying statistic information
@@ -717,7 +680,7 @@ public:
     //@}
 
     //! Increments the triangle counter for the current frame
-    void            AddStatisticTriangle(int nb);
+    void            AddStatisticTriangle(int count);
     //! Returns the number of triangles in current frame
     int             GetStatisticTriangle();
 
@@ -778,7 +741,7 @@ public:
     // Objects
 
     //! Print debug info about an object
-    void            DebugObject(int rank);
+    void            DebugObject(int objRank);
 
     //! Creates a new object and returns its rank
     int             CreateObject();
@@ -844,7 +807,7 @@ public:
 
     //! Detects the target object that is selected with the mouse
     /** Returns the rank of the object or -1. */
-    int             DetectObject(Math::Point mouse);
+    int             DetectObject(Math::Point mouse, Math::Vector& targetPos, bool terrain = false);
 
     //! Creates a shadow for the given object
     void            CreateShadowSpot(int objRank);
@@ -904,8 +867,7 @@ public:
     void            SetMaterial(const Material& mat);
 
     //! Specifies the location and direction of view
-    void            SetViewParams(const Math::Vector& eyePt, const Math::Vector& lookatPt,
-                                  const Math::Vector& upVec, float eyeDistance);
+    void SetViewParams(const Math::Vector &eyePt, const Math::Vector &lookatPt, const Math::Vector &upVec);
 
     //! Loads texture, creating it if not already present
     Texture         LoadTexture(const std::string& name);
@@ -942,10 +904,13 @@ public:
     void            SetTexture(const Texture& tex, int stage = 0);
 
     //! Deletes the given texture, unloading it and removing from cache
-    void            DeleteTexture(const std::string& name);
+    void            DeleteTexture(const std::string& texName);
     //! Deletes the given texture, unloading it and removing from cache
     void            DeleteTexture(const Texture& tex);
 
+    //! Creates or updates the given texture with given image data
+    void            CreateOrUpdateTexture(const std::string& texName, CImage* img);
+
     //! Empties the texture cache
     void            FlushTextureCache();
 
@@ -1096,6 +1061,8 @@ public:
     bool            IsShadowMappingQualitySupported();
     void            SetShadowMappingQuality(bool value);
     bool            GetShadowMappingQuality();
+    void            SetTerrainShadows(bool value);
+    bool            GetTerrainShadows();
     //@}
 
     //@{
@@ -1160,6 +1127,12 @@ public:
     EngineMouseType GetMouseType();
     //@}
 
+    //@{
+    //! Management of pause blur
+    void            SetPauseBlurEnabled(bool enable);
+    bool            GetPauseBlurEnabled();
+    //@}
+
     //! Returns the view matrix
     const Math::Matrix& GetMatView();
     //! Returns the camera center point
@@ -1179,9 +1152,38 @@ public:
     //! Updates the scene after a change of parameters
     void            ApplyChange();
 
+    void            ClearDisplayCrashSpheres();
+    void            AddDisplayCrashSpheres(const std::vector<Math::Sphere>& crashSpheres);
+
+    void            SetDebugLights(bool debugLights);
+    bool            GetDebugLights();
+    void            DebugDumpLights();
+
+    void            SetDebugResources(bool debugResources);
+    bool            GetDebugResources();
+
+    void            SetDebugGoto(bool debugGoto);
+    bool            GetDebugGoto();
+    void            AddDebugGotoLine(std::vector<Gfx::VertexCol> line);
+    void            SetDebugGotoBitmap(std::unique_ptr<CImage> debugImage);
+
+    void            SetWindowCoordinates();
+    void            SetInterfaceCoordinates();
+
+    void            EnablePauseBlur();
+    void            DisablePauseBlur();
+
 protected:
+    //! Resets some states and flushes textures after device was changed (e.g. resoulution changed)
+    /** Instead of calling this directly, send EVENT_RESOLUTION_CHANGED event **/
+    void            ResetAfterVideoConfigChanged();
+
     //! Prepares the interface for 3D scene
     void        Draw3DScene();
+    //! Capture the 3D scene for pause blur
+    void        Capture3DScene();
+    //! Draw the 3D scene capured for pause blur
+    void        DrawCaptured3DScene();
     //! Renders shadow map
     void        RenderShadowMap();
     //! Enables or disables shadow mapping
@@ -1215,17 +1217,18 @@ protected:
     //! Draws the mouse cursor
     void        DrawMouse();
     //! Draw part of mouse cursor sprite
-    void        DrawMouseSprite(Math::Point pos, Math::Point dim, int icon);
+    void        DrawMouseSprite(Math::IntPoint pos, Math::IntPoint size, int icon);
     //! Draw statistic texts
     void        DrawStats();
     //! Draw mission timer
     void        DrawTimer();
+    void        DrawCrashSpheres();
 
     //! Creates a new tier 2 object (texture)
     EngineBaseObjTexTier&  AddLevel2(EngineBaseObject& p1, const std::string& tex1Name, const std::string& tex2Name);
     //! Creates a new tier 3 object (data)
     EngineBaseObjDataTier& AddLevel3(EngineBaseObjTexTier &p3, EngineTriangleType type,
-                                     const Material& mat, int state);
+                                     const Material& material, int state);
 
     //! Create texture and add it to cache
     Texture CreateTexture(const std::string &texName, const TextureCreateParams &params, CImage* image = nullptr);
@@ -1240,7 +1243,7 @@ protected:
     bool        GetBBox2D(int objRank, Math::Point& min, Math::Point& max);
 
     //! Detects whether the mouse is in a triangle.
-    bool        DetectTriangle(Math::Point mouse, VertexTex2* triangle, int objRank, float& dist);
+    bool        DetectTriangle(Math::Point mouse, VertexTex2* triangle, int objRank, float& dist, Math::Vector& pos);
 
     //! Transforms a 3D point (x, y, z) in 2D space (x, y, -) of the window
     /** The coordinated p2D.z gives the distance. */
@@ -1271,6 +1274,10 @@ protected:
     };
     static void WriteScreenShotThread(std::unique_ptr<WriteScreenShotData> data);
 
+    //! Reloads all textures
+    /** This additionally sends EVENT_RELOAD_TEXTURES to reload all textures not maintained by CEngine **/
+    void ReloadAllTextures();
+
 protected:
     CApplication*     m_app;
     CSystemUtils*     m_systemUtils;
@@ -1285,7 +1292,6 @@ protected:
     std::unique_ptr<CCloud>           m_cloud;
     std::unique_ptr<CLightning>       m_lightning;
     std::unique_ptr<CPlanet>          m_planet;
-    std::unique_ptr<CPauseManager>    m_pause;
     std::unique_ptr<CPyroManager> m_pyroManager;
 
     //! Last encountered error
@@ -1380,12 +1386,12 @@ protected:
     float           m_terrainVision;
     bool            m_backForce;
     float           m_tracePrecision;
+    bool            m_pauseBlurEnabled;
 
     bool            m_dirty;
     bool            m_fog;
     float           m_particleDensity;
     float           m_clippingDistance;
-    float           m_lastClippingDistance;
     bool            m_lightMode;
     bool            m_editIndentMode;
     int             m_editIndentValue;
@@ -1420,6 +1426,8 @@ protected:
     int m_offscreenShadowRenderingResolution;
     //! true enables higher quality shadows
     bool m_qualityShadows;
+    //! true enables casting shadows by terrain
+    bool m_terrainShadows;
     //! Shadow color
     float m_shadowColor;
     //! Shadow range
@@ -1436,12 +1444,8 @@ protected:
      *  so are disabled for subsequent load calls. */
     std::set<std::string> m_texBlacklist;
 
-    //! Mouse cursor definitions
-    EngineMouse     m_mice[ENG_MOUSE_COUNT];
     //! Texture with mouse cursors
     Texture         m_miceTexture;
-    //! Size of mouse cursor
-    Math::Point     m_mouseSize;
     //! Type of mouse cursor
     EngineMouseType m_mouseType;
 
@@ -1459,10 +1463,27 @@ protected:
 
     bool            m_debugLights;
     bool            m_debugDumpLights;
+    bool            m_debugCrashSpheres = false;
+    bool            m_debugResources = false;
+    bool            m_debugGoto = false;
 
     std::string     m_timerText;
 
     std::unordered_map<std::string, int> m_staticMeshBaseObjects;
+
+    std::vector<Math::Sphere> m_displayCrashSpheres;
+    std::vector<std::vector<VertexCol>> m_displayGoto;
+    std::unique_ptr<CImage> m_displayGotoImage;
+
+    //! Pause the animation updates
+    bool            m_pause = false;
+
+    //! true means that current 3D scene was captured and is not to be rendered again
+    bool            m_worldCaptured = false;
+    //! true means that currently rendered world is to be captured
+    bool            m_captureWorld = false;
+    //! Texture with captured 3D world
+    Texture         m_capturedWorldTexture;
 };
 
 
