@@ -1,6 +1,6 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * Copyright (C) 2001-2016, Daniel Roux, EPSITEC SA & TerranovaTeam
  * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
@@ -162,9 +162,9 @@ enum ParticlePhase
 
 struct Particle
 {
-    bool            used = false;      // TRUE -> particle used
+    bool            used = false;      //!< true if this channel is used, false if not
     bool            ray = false;       // TRUE -> ray with goal
-    unsigned short  uniqueStamp = 0;    // unique mark
+    unsigned short  uniqueStamp = 0;    //!< unique marker added to particle channel ID to make sure this is still the same particle
     short           sheet = 0;      // sheet (0..n)
     ParticleType    type = {};       // type PARTI*
     ParticlePhase   phase = {};      // phase PARPH*
@@ -279,7 +279,7 @@ public:
     void        SetPhase(int channel, ParticlePhase phase, float duration);
 
     //! Returns the position of the particle
-    bool        GetPosition(int channel, Math::Vector &pos);
+    Math::Vector GetPosition(int channel);
 
     //! Returns the color if you're in the fog or black if you're not
     Color       GetFogColor(Math::Vector pos);
@@ -291,14 +291,18 @@ public:
     //! Draws all the particles
     void        DrawParticle(int sheet);
 
-    //! Writes a file containing all the tire tracks
-    bool        WriteWheelTrace(const char *filename, int width, int height, Math::Vector dl, Math::Vector ur);
+    //! Checks if given particle channel still exists
+    bool        ParticleExists(int channel);
 
 protected:
     //! Removes a particle of given rank
     void        DeleteRank(int rank);
-    //! Check a channel number
-    bool        CheckChannel(int &channel);
+    /**
+     * \brief Adapts the channel so it can be used as an offset in m_particle
+     * \param channel Channel number to process, will be modified to be index of particle in m_particle
+     * \throw std::runtime_error if this particle does not exist any more
+     **/
+    void        GetRankFromChannel(int &channel);
     //! Draws a triangular particle
     void        DrawParticleTriangle(int i);
     //! Draw a normal particle
