@@ -1,6 +1,6 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * Copyright (C) 2001-2016, Daniel Roux, EPSITEC SA & TerranovaTeam
  * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
@@ -192,6 +192,22 @@ void CTerrain::AddMaterial(int id, const std::string& texName, const Math::Point
 }
 
 
+// values from original bitmap palette
+const std::map<TerrainRes, Gfx::IntColor> RESOURCE_PALETTE = {
+    {TR_STONE,   Gfx::IntColor(255,   0, 0)},
+    {TR_URANIUM, Gfx::IntColor(255, 255, 0)},
+    {TR_POWER,   Gfx::IntColor(  0, 255, 0)},
+    {TR_KEY_A,   Gfx::IntColor(  0, 204, 0)},
+    {TR_KEY_B,   Gfx::IntColor( 51, 204, 0)},
+    {TR_KEY_C,   Gfx::IntColor(102, 204, 0)},
+    {TR_KEY_D,   Gfx::IntColor(153, 204, 0)}
+};
+
+Gfx::IntColor ResourceToColor(TerrainRes res)
+{
+    return RESOURCE_PALETTE.at(res);
+}
+
 /**
  * The image must be 24 bits/pixel and grayscale and dx x dy in size
  * with dx = dy = (mosaic*brick)+1 */
@@ -224,21 +240,11 @@ bool CTerrain::LoadResources(const std::string& fileName)
             Gfx::IntColor pixel = img.GetPixelInt(Math::IntPoint(x, size - y - 1));
             TerrainRes res = TR_NULL;
 
-            // values from original bitmap palette
-            if      (pixel.r == 255 && pixel.g ==   0 && pixel.b == 0)
-                res = TR_STONE;
-            else if (pixel.r == 255 && pixel.g == 255 && pixel.b == 0)
-                res = TR_URANIUM;
-            else if (pixel.r ==   0 && pixel.g == 255 && pixel.b == 0)
-                res = TR_POWER;
-            else if (pixel.r ==   0 && pixel.g == 204 && pixel.b == 0)
-                res = TR_KEY_A;
-            else if (pixel.r ==  51 && pixel.g == 204 && pixel.b == 0)
-                res = TR_KEY_B;
-            else if (pixel.r == 102 && pixel.g == 204 && pixel.b == 0)
-                res = TR_KEY_C;
-            else if (pixel.r == 153 && pixel.g == 204 && pixel.b == 0)
-                res = TR_KEY_D;
+            for (const auto& it : RESOURCE_PALETTE)
+            {
+                if (pixel.r == it.second.r && pixel.g == it.second.g && pixel.b == it.second.b)
+                    res = it.first;
+            }
 
             m_resources[x+size*y] = static_cast<unsigned char>(res);
         }
@@ -247,13 +253,13 @@ bool CTerrain::LoadResources(const std::string& fileName)
     return true;
 }
 
-TerrainRes CTerrain::GetResource(const Math::Vector &p)
+TerrainRes CTerrain::GetResource(const Math::Vector &pos)
 {
     if (m_resources.empty())
         return TR_NULL;
 
-    int x = static_cast<int>((p.x + (m_mosaicCount*m_brickCount*m_brickSize)/2.0f)/m_brickSize);
-    int y = static_cast<int>((p.z + (m_mosaicCount*m_brickCount*m_brickSize)/2.0f)/m_brickSize);
+    int x = static_cast<int>((pos.x + (m_mosaicCount*m_brickCount*m_brickSize)/2.0f)/m_brickSize);
+    int y = static_cast<int>((pos.z + (m_mosaicCount*m_brickCount*m_brickSize)/2.0f)/m_brickSize);
 
     if ( x < 0 || x > m_mosaicCount*m_brickCount ||
          y < 0 || y > m_mosaicCount*m_brickCount )
@@ -1677,16 +1683,16 @@ bool CTerrain::DeleteBuildingLevel(Math::Vector center)
     return false;
 }
 
-float CTerrain::GetBuildingFactor(const Math::Vector &p)
+float CTerrain::GetBuildingFactor(const Math::Vector &pos)
 {
     for (int i = 0; i < static_cast<int>( m_buildingLevels.size() ); i++)
     {
-        if ( p.x < m_buildingLevels[i].bboxMinX ||
-             p.x > m_buildingLevels[i].bboxMaxX ||
-             p.z < m_buildingLevels[i].bboxMinZ ||
-             p.z > m_buildingLevels[i].bboxMaxZ )  continue;
+        if ( pos.x < m_buildingLevels[i].bboxMinX ||
+             pos.x > m_buildingLevels[i].bboxMaxX ||
+             pos.z < m_buildingLevels[i].bboxMinZ ||
+             pos.z > m_buildingLevels[i].bboxMaxZ )  continue;
 
-        float dist = Math::DistanceProjected(p, m_buildingLevels[i].center);
+        float dist = Math::DistanceProjected(pos, m_buildingLevels[i].center);
 
         if (dist <= m_buildingLevels[i].max)
             return m_buildingLevels[i].factor;
@@ -1730,9 +1736,9 @@ void CTerrain::AdjustBuildingLevel(Math::Vector &p)
     }
 }
 
-float CTerrain::GetHardness(const Math::Vector &p)
+float CTerrain::GetHardness(const Math::Vector &pos)
 {
-    float factor = GetBuildingFactor(p);
+    float factor = GetBuildingFactor(pos);
     if (factor != 1.0f) return 1.0f;  // on building level
 
     if (m_materialPoints.empty()) return m_defaultHardness;
@@ -1741,8 +1747,8 @@ float CTerrain::GetHardness(const Math::Vector &p)
 
     int x, y;
 
-    x = static_cast<int>((p.x+dim)/m_brickSize);
-    y = static_cast<int>((p.z+dim)/m_brickSize);
+    x = static_cast<int>((pos.x+dim)/m_brickSize);
+    y = static_cast<int>((pos.z+dim)/m_brickSize);
 
     if ( x < 0 || x > m_mosaicCount*m_brickCount ||
          y < 0 || y > m_mosaicCount*m_brickCount )  return m_defaultHardness;
