@@ -1,6 +1,6 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * Copyright (C) 2001-2016, Daniel Roux, EPSITEC SA & TerranovaTeam
  * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
@@ -20,6 +20,8 @@
 
 #include "script/script.h"
 
+#include "CBot/CBot.h"
+
 #include "common/restext.h"
 #include "common/stringutils.h"
 
@@ -42,7 +44,7 @@
 #include "ui/controls/interface.h"
 #include "ui/controls/list.h"
 
-
+#include <libintl.h>
 
 const int CBOT_IPF = 100;       // CBOT: default number of instructions / frame
 
@@ -69,11 +71,8 @@ CScript::CScript(COldObject* object)
     m_bRun = false;
     m_bStepMode = false;
     m_bCompile = false;
-    m_title[0] = 0;
-    m_mainFunction[0] = 0;
     m_cursor1 = 0;
     m_cursor2 = 0;
-    m_filename[0] = 0;
 }
 
 // Object's destructor.
@@ -155,71 +154,62 @@ bool CScript::IsEmpty()
 
 bool CScript::CheckToken()
 {
-    CBotToken*  bt;
-    CBotToken*  allBt;
-    CBotString  bs;
-    const char* token;
-    int         error, cursor1, cursor2, i;
-    char        used[100];
-
     if ( !m_object->GetCheckToken() )  return true;
 
-    m_error = 0;
-    m_title[0] = 0;
-    m_mainFunction[0] = 0;
-    m_token[0] = 0;
+    m_error = CBot::CBotNoErr;
+    m_title.clear();
+    m_mainFunction.clear();
+    m_token.clear();
     m_bCompile = false;
 
-    for ( i=0 ; i<m_main->GetObligatoryToken() ; i++ )
-    {
-        used[i] = 0;  // token not used
-    }
+    std::map<std::string, int> used;
+    std::map<std::string, int> cursor1;
+    std::map<std::string, int> cursor2;
 
-    allBt = CBotToken::CompileTokens(m_script.get(), error);
-    bt = allBt;
+    auto tokens = CBot::CBotToken::CompileTokens(m_script.get());
+    CBot::CBotToken* bt = tokens.get();
     while ( bt != nullptr )
     {
-        bs = bt->GetString();
-        token = bs;
+        std::string token = bt->GetString();
 
-        cursor1 = bt->GetStart();
-        cursor2 = bt->GetEnd();
+        // Store only the last occurrence of the token
+        cursor1[token] = bt->GetStart();
+        cursor2[token] = bt->GetEnd();
 
-        i = m_main->IsObligatoryToken(token);
-        if ( i != -1 )
-        {
-            used[i] = 1;  // token used
-        }
-
-        if ( !m_main->IsProhibitedToken(token) )
-        {
-            m_error = ERR_PROHIBITEDTOKEN;
-            m_cursor1 = cursor1;
-            m_cursor2 = cursor2;
-            strcpy(m_title, "<prohibited>");
-            m_mainFunction[0] = 0;
-            CBotToken::Delete(allBt);
-            return false;
-        }
+        used[token]++;
 
         bt = bt->GetNext();
     }
 
-    // At least once every obligatory instruction?
-    for ( i=0 ; i<m_main->GetObligatoryToken() ; i++ )
+    for (const auto& it : m_main->GetObligatoryTokenList())
     {
-        if ( used[i] == 0 )  // token not used?
+        Error error = ERR_OK;
+        int allowed = 0;
+        if (it.second.max >= 0 && used[it.first] > it.second.max)
         {
-            strcpy(m_token, m_main->GetObligatoryToken(i));
-            m_error = ERR_OBLIGATORYTOKEN;
-            strcpy(m_title, "<obligatory>");
-            m_mainFunction[0] = 0;
-            CBotToken::Delete(allBt);
+            error = ERR_PROHIBITEDTOKEN;
+            allowed = it.second.max;
+            m_cursor1 = cursor1[it.first];
+            m_cursor2 = cursor2[it.first];
+        }
+        if (it.second.min >= 0 && used[it.first] < it.second.min)
+        {
+            error = ERR_OBLIGATORYTOKEN;
+            allowed = it.second.min;
+        }
+
+        if (error != ERR_OK)
+        {
+            m_token = it.first;
+            m_tokenUsed = used[it.first];
+            m_tokenAllowed = allowed;
+            m_error = static_cast<CBot::CBotError>(error);
+            m_title = "<incorrect instructions>";
+            m_mainFunction.clear();
             return false;
         }
     }
 
-    CBotToken::Delete(allBt);
     return true;
 }
 
@@ -227,15 +217,14 @@ bool CScript::CheckToken()
 
 bool CScript::Compile()
 {
-    CBotStringArray liste;
-    int             i;
-    const char*     p;
+    std::vector<std::string> functionList;
+    std::string     p;
 
-    m_error = 0;
+    m_error = CBot::CBotNoErr;
     m_cursor1 = 0;
     m_cursor2 = 0;
-    m_title[0] = 0;
-    m_mainFunction[0] = 0;
+    m_title.clear();
+    m_mainFunction.clear();
     m_bCompile = false;
 
     if ( IsEmpty() )  // program exist?
@@ -246,40 +235,24 @@ bool CScript::Compile()
 
     if (m_botProg == nullptr)
     {
-        m_botProg = MakeUnique<CBotProgram>(m_object->GetBotVar());
+        m_botProg = MakeUnique<CBot::CBotProgram>(m_object->GetBotVar());
     }
 
-    if ( m_botProg->Compile(m_script.get(), liste, this) )
+    if ( m_botProg->Compile(m_script.get(), functionList, this) )
     {
-        if ( liste.GetSize() == 0 )
+        if (functionList.empty())
         {
-            strcpy(m_title, "<extern missing>");
-            m_mainFunction[0] = 0;
+            m_title = "<extern missing>";
+            m_mainFunction.clear();
         }
         else
         {
-            p = liste[0];
-            i = 0;
-            bool titleDone = false;
-            while ( true )
+            m_mainFunction = functionList[0];
+            m_title = m_mainFunction;
+            if (m_title.length() >= 20)
             {
-                if ( p[i] == 0 || p[i] == '(' )  break;
-                if ( i >= 20 && !titleDone )
-                {
-                    m_title[i+0] = '.';
-                    m_title[i+1] = '.';
-                    m_title[i+2] = '.';
-                    m_title[i+3] = 0;
-                    titleDone = true;
-                }
-                if(!titleDone)
-                    m_title[i] = p[i];
-                m_mainFunction[i] = p[i];
-                i ++;
+                m_title = m_title.substr(0, 20)+"...";
             }
-            if(!titleDone)
-                m_title[i] = 0;
-            m_mainFunction[i] = p[i];
         }
         m_bCompile = true;
         return true;
@@ -297,8 +270,8 @@ bool CScript::Compile()
         {
             m_cursor1 = m_cursor2 = 0;
         }
-        strcpy(m_title, "<error>");
-        m_mainFunction[0] = 0;
+        m_title = "<error>";
+        m_mainFunction.clear();
         return false;
     }
 }
@@ -306,9 +279,9 @@ bool CScript::Compile()
 
 // Returns the title of the script.
 
-void CScript::GetTitle(char* buffer)
+const std::string& CScript::GetTitle()
 {
-    strcpy(buffer, m_title);
+    return m_title;
 }
 
 
@@ -331,9 +304,9 @@ bool CScript::Run()
 {
     if (m_botProg == nullptr)  return false;
     if ( m_script == nullptr || m_len == 0 )  return false;
-    if ( m_mainFunction[0] == 0 ) return false;
+    if ( m_mainFunction.empty() ) return false;
 
-    if ( !m_botProg->Start(m_mainFunction) )  return false;
+    if ( !m_botProg->Start(m_mainFunction.c_str()) )  return false;
 
     m_bRun = true;
     m_bContinue = false;
@@ -485,8 +458,7 @@ bool CScript::IsContinue()
 
 bool CScript::GetCursor(int &cursor1, int &cursor2)
 {
-    const char* funcName;
-
+    std::string funcName;
     cursor1 = cursor2 = 0;
 
     if (m_botProg == nullptr)  return false;
@@ -505,85 +477,71 @@ bool CScript::GetCursor(int &cursor1, int &cursor2)
 
 // Put of the variables in a list.
 
-void PutList(const char *baseName, bool bArray, CBotVar *var, Ui::CList *list, int &rankList)
+void PutList(const std::string& baseName, bool bArray, CBot::CBotVar *var, Ui::CList *list, int &rankList, std::set<CBot::CBotVar*>& previous)
 {
-    CBotString  bs;
-    CBotVar     *svar, *pStatic;
-    char        varName[100];
-    char        buffer[100];
-    const char  *p;
-    int         index, type;
-
-    if ( var == nullptr && baseName[0] != 0 )
+    if ( var == nullptr && !baseName.empty() )
     {
-        sprintf(buffer, "%s = null;", baseName);
-        list->SetItemName(rankList++, buffer);
+        list->SetItemName(rankList++, StrUtils::Format("%s = null;", baseName.c_str()).c_str());
         return;
     }
 
-    index = 0;
-    while ( var != nullptr )
+    int index = 0;
+    while (var != nullptr)
     {
-        var->Maj(nullptr, false);
-        pStatic = var->GetStaticVar();  // finds the static element
+        var->Update(nullptr);
+        CBot::CBotVar* pStatic = var->GetStaticVar();  // finds the static element
 
-        bs = pStatic->GetName();  // variable name
-        p = bs;
-//?     if ( strcmp(p, "this") == 0 )
-//?     {
-//?         var = var->GetNext();
-//?         continue;
-//?     }
-
-        if ( baseName[0] == 0 )
+        std::string varName;
+        if (baseName.empty())
         {
-            sprintf(varName, "%s", p);
+            varName = StrUtils::Format("%s", pStatic->GetName().c_str());
         }
         else
         {
-            if ( bArray )
+            if (bArray)
             {
-                sprintf(varName, "%s[%d]", baseName, index);
+                varName = StrUtils::Format("%s[%d]", baseName.c_str(), index);
             }
             else
             {
-                sprintf(varName, "%s.%s", baseName, p);
+                varName = StrUtils::Format("%s.%s", baseName.c_str(), pStatic->GetName().c_str());
             }
         }
 
-        type = pStatic->GetType();
-
-        if ( type < CBotTypBoolean )
+        if (previous.find(pStatic) != previous.end())
         {
-            CBotString  value;
-            value = pStatic->GetValString();
-            p = value;
-            sprintf(buffer, "%s = %s;", varName, p);
-            list->SetItemName(rankList++, buffer);
-        }
-        else if ( type == CBotTypString )
-        {
-            CBotString  value;
-            value = pStatic->GetValString();
-            p = value;
-            sprintf(buffer, "%s = \"%s\";", varName, p);
-            list->SetItemName(rankList++, buffer);
-        }
-        else if ( type == CBotTypArrayPointer )
-        {
-            svar = pStatic->GetItemList();
-            PutList(varName, true, svar, list, rankList);
-        }
-        else if ( type == CBotTypClass   ||
-                  type == CBotTypPointer )
-        {
-            svar = pStatic->GetItemList();
-            PutList(varName, false, svar, list, rankList);
+            list->SetItemName(rankList++, StrUtils::Format("%s = [circular reference]", varName.c_str()));
         }
         else
         {
-            sprintf(buffer, "%s = ?;", varName);
-            list->SetItemName(rankList++, buffer);
+            int type = pStatic->GetType();
+
+            if (type <= CBot::CBotTypBoolean)
+            {
+                list->SetItemName(rankList++, StrUtils::Format("%s = %s;", varName.c_str(), pStatic->GetValString().c_str()));
+            }
+            else if (type == CBot::CBotTypString)
+            {
+                list->SetItemName(rankList++, StrUtils::Format("%s = \"%s\";", varName.c_str(), pStatic->GetValString().c_str()));
+            }
+            else if (type == CBot::CBotTypArrayPointer)
+            {
+                previous.insert(pStatic);
+                PutList(varName, true, pStatic->GetItemList(), list, rankList, previous);
+                previous.erase(pStatic);
+            }
+            else if (type == CBot::CBotTypClass ||
+                     type == CBot::CBotTypPointer)
+            {
+                previous.insert(pStatic);
+                PutList(varName, false, pStatic->GetItemList(), list, rankList, previous);
+                previous.erase(pStatic);
+            }
+            else
+            {
+                //list->SetItemName(rankList++, StrUtils::Format("%s = ?;", varName.c_str()));
+                assert(false);
+            }
         }
 
         index ++;
@@ -595,8 +553,8 @@ void PutList(const char *baseName, bool bArray, CBotVar *var, Ui::CList *list, i
 
 void CScript::UpdateList(Ui::CList* list)
 {
-    CBotVar     *var;
-    const char  *progName, *funcName;
+    CBot::CBotVar     *var;
+    std::string progName, funcName;
     int         total, select, level, cursor1, cursor2, rank;
 
     if (m_botProg == nullptr) return;
@@ -606,17 +564,19 @@ void CScript::UpdateList(Ui::CList* list)
 
     list->Flush();  // empty list
     m_botProg->GetRunPos(progName, cursor1, cursor2);
-    if ( progName == nullptr )  return;
+    if ( progName.empty() )  return;
 
     level = 0;
     rank  = 0;
+    std::set<CBot::CBotVar*> previous;
     while ( true )
     {
         var = m_botProg->GetStackVars(funcName, level--);
         if ( funcName != progName )  break;
 
-        PutList("", false, var, list, rank);
+        PutList("", false, var, list, rank, previous);
     }
+    assert(previous.empty());
 
     if ( total == list->GetTotal() )  // same total?
     {
@@ -641,12 +601,11 @@ void CScript::ColorizeScript(Ui::CEdit* edit, int rangeStart, int rangeEnd)
     std::string text = std::string(edit->GetText(), edit->GetMaxChar());
     text = text.substr(rangeStart, rangeEnd-rangeStart);
 
-    int error;
-    CBotToken* bt = CBotToken::CompileTokens(text.c_str(), error);
+    auto tokens = CBot::CBotToken::CompileTokens(text.c_str());
+    CBot::CBotToken* bt = tokens.get();
     while ( bt != nullptr )
     {
-        CBotString bs = bt->GetString();
-        const char* token = bs;
+        std::string token = bt->GetString();
         int type = bt->GetType();
 
         int cursor1 = bt->GetStart();
@@ -658,35 +617,31 @@ void CScript::ColorizeScript(Ui::CEdit* edit, int rangeStart, int rangeEnd)
         cursor2 += rangeStart;
 
         Gfx::FontHighlight color = Gfx::FONT_HIGHLIGHT_NONE;
-        if ((type == TokenTypVar || (type >= TokenKeyWord && type < TokenKeyWord+100)) && IsType(token)) // types (basic types are TokenKeyWord, classes are TokenTypVar)
+        if ((type == CBot::TokenTypVar || (type >= CBot::TokenKeyWord && type < CBot::TokenKeyWord+100)) && IsType(token.c_str())) // types (basic types are TokenKeyWord, classes are TokenTypVar)
         {
             color = Gfx::FONT_HIGHLIGHT_TYPE;
         }
-        else if (type == TokenTypVar && IsFunction(token)) // functions
+        else if (type == CBot::TokenTypVar && IsFunction(token.c_str())) // functions
         {
             color = Gfx::FONT_HIGHLIGHT_TOKEN;
         }
-        else if (type == TokenTypVar && strcmp(token, "this") == 0) // this
+        else if (type == CBot::TokenTypVar && (token == "this" || token == "super")) // this, super
         {
             color = Gfx::FONT_HIGHLIGHT_THIS;
         }
-        else if (type >= TokenKeyWord && type < TokenKeyWord+100) // builtin keywords
+        else if (type >= CBot::TokenKeyWord && type < CBot::TokenKeyWord+100) // builtin keywords
         {
             color = Gfx::FONT_HIGHLIGHT_KEYWORD;
         }
-        else if (type >= TokenKeyDeclare && type < TokenKeyDeclare+100) // TODO: no idea :P seems to never happen
-        {
-            color = Gfx::FONT_HIGHLIGHT_TYPE;
-        }
-        else if (type >= TokenKeyVal && type < TokenKeyVal+100) // true, false, null, nan
+        else if (type >= CBot::TokenKeyVal && type < CBot::TokenKeyVal+100) // true, false, null, nan
         {
             color = Gfx::FONT_HIGHLIGHT_CONST;
         }
-        else if (type == TokenTypDef) // constants (object types etc.)
+        else if (type == CBot::TokenTypDef) // constants (object types etc.)
         {
             color = Gfx::FONT_HIGHLIGHT_CONST;
         }
-        else if (type == TokenTypString || type == TokenTypNum) // string literals and numbers
+        else if (type == CBot::TokenTypString || type == CBot::TokenTypNum) // string literals and numbers
         {
             color = Gfx::FONT_HIGHLIGHT_STRING;
         }
@@ -696,8 +651,6 @@ void CScript::ColorizeScript(Ui::CEdit* edit, int rangeStart, int rangeEnd)
 
         bt = bt->GetNext();
     }
-
-    CBotToken::Delete(bt);
 }
 
 
@@ -822,13 +775,28 @@ void CScript::GetError(std::string& error)
     }
     else
     {
-        if ( m_error == ERR_OBLIGATORYTOKEN )
+        if (m_error == static_cast<CBot::CBotError>(ERR_OBLIGATORYTOKEN))
         {
-            std::string s;
-            GetResource(RES_ERR, m_error, s);
-            error = StrUtils::Format(s.c_str(), m_token);
+            error = StrUtils::Format(ngettext(
+                "You have to use \"%1$s\" at least once in this exercise (used: %2$d)",
+                "You have to use \"%1$s\" at least %3$d times in this exercise (used: %2$d)",
+                m_tokenAllowed), m_token.c_str(), m_tokenUsed, m_tokenAllowed);
         }
-        else if ( m_error < 1000 )
+        else if (m_error == static_cast<CBot::CBotError>(ERR_PROHIBITEDTOKEN))
+        {
+            if (m_tokenAllowed == 0)
+            {
+                error = StrUtils::Format(gettext("You cannot use \"%s\" in this exercise (used: %d)"), m_token.c_str(), m_tokenUsed);
+            }
+            else
+            {
+                error = StrUtils::Format(ngettext(
+                    "You have to use \"%1$s\" at most once in this exercise (used: %2$d)",
+                    "You have to use \"%1$s\" at most %3$d times in this exercise (used: %2$d)",
+                    m_tokenAllowed), m_token.c_str(), m_tokenUsed, m_tokenAllowed);
+            }
+        }
+        else if (m_error < 1000)
         {
             GetResource(RES_ERR, m_error, error);
         }
@@ -848,7 +816,6 @@ void CScript::New(Ui::CEdit* edit, const char* name)
     char    text[100];
     char    script[500];
     char    buffer[500];
-    char    *sf;
     int     cursor1, cursor2, len, i, j;
 
     std::string resStr;
@@ -883,12 +850,11 @@ void CScript::New(Ui::CEdit* edit, const char* name)
     edit->ShowSelect();
     m_interface->SetFocus(edit);
 
-    sf = m_main->GetScriptFile();
-    if ( sf[0] != 0 )  // Load an empty program specific?
+    std::string sf = m_main->GetScriptFile();
+    if ( !sf.empty() )  // Load an empty program specific?
     {
-        std::string filename = sf;
         CInputStream stream;
-        stream.open(filename);
+        stream.open(sf);
 
         if (stream.is_open())
         {
@@ -1015,9 +981,9 @@ bool CScript::ReadStack(FILE *file)
 {
     int     nb;
 
-    fRead(&nb, sizeof(int), 1, file);
-    fRead(&m_ipf, sizeof(int), 1, file);
-    fRead(&m_errMode, sizeof(int), 1, file);
+    CBot::fRead(&nb, sizeof(int), 1, file);
+    CBot::fRead(&m_ipf, sizeof(int), 1, file);
+    CBot::fRead(&m_errMode, sizeof(int), 1, file);
 
     if (m_botProg == nullptr) return false;
     if ( !m_botProg->RestoreState(file) )  return false;
@@ -1034,9 +1000,9 @@ bool CScript::WriteStack(FILE *file)
     int     nb;
 
     nb = 2;
-    fWrite(&nb, sizeof(int), 1, file);
-    fWrite(&m_ipf, sizeof(int), 1, file);
-    fWrite(&m_errMode, sizeof(int), 1, file);
+    CBot::fWrite(&nb, sizeof(int), 1, file);
+    CBot::fWrite(&m_ipf, sizeof(int), 1, file);
+    CBot::fWrite(&m_errMode, sizeof(int), 1, file);
 
     return m_botProg->SaveState(file);
 }
@@ -1054,12 +1020,12 @@ bool CScript::Compare(CScript* other)
 
 // Management of the file name when the script is saved.
 
-void CScript::SetFilename(char *filename)
+void CScript::SetFilename(const std::string& filename)
 {
-    strcpy(m_filename, filename);
+    m_filename = filename;
 }
 
-char* CScript::GetFilename()
+const std::string& CScript::GetFilename()
 {
     return m_filename;
 }
