@@ -1,6 +1,6 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * Copyright (C) 2001-2016, Daniel Roux, EPSITEC SA & TerranovaTeam
  * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
@@ -27,8 +27,6 @@
 #include <iomanip>
 #include <sstream>
 
-#include <boost/filesystem.hpp>
-
 
 CSoundInterface::CSoundInterface()
 {
@@ -54,7 +52,7 @@ void CSoundInterface::CacheAll()
     }
 }
 
-void CSoundInterface::AddMusicFiles()
+void CSoundInterface::CacheCommonMusic()
 {
     CacheMusic("music/Intro1.ogg");
     CacheMusic("music/Intro2.ogg");
@@ -153,7 +151,7 @@ bool CSoundInterface::StopAll()
     return true;
 }
 
-bool CSoundInterface::MuteAll(bool bMute)
+bool CSoundInterface::MuteAll(bool mute)
 {
     return true;
 }
