@@ -1,6 +1,6 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * Copyright (C) 2001-2016, Daniel Roux, EPSITEC SA & TerranovaTeam
  * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
@@ -24,12 +24,15 @@
 
 #pragma once
 
-#include "math/vector.h"
-
 #include "sound/sound_type.h"
 
 #include <string>
 
+namespace Math
+{
+struct Vector;
+} // namespace Math
+
 /*!
  * Maximum possible audio volume
  */
@@ -70,7 +73,7 @@ public:
     void CacheAll();
 
     /** Function called to add all music files to list */
-    void AddMusicFiles();
+    void CacheCommonMusic();
 
     /** Function called to cache sound effect file.
      *  This function is called by plugin interface for each file.
