@@ -1,6 +1,6 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * Copyright (C) 2001-2016, Daniel Roux, EPSITEC SA & TerranovaTeam
  * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
@@ -19,11 +19,9 @@
 
 #pragma once
 
-
 #include "ui/controls/control.h"
 
 #include "common/event.h"
-#include "common/misc.h"
 
 namespace Ui
 {
@@ -41,4 +39,4 @@ public:
 };
 
 
-}
+} // namespace Ui
