@@ -1,6 +1,6 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * Copyright (C) 2001-2016, Daniel Roux, EPSITEC SA & TerranovaTeam
  * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
@@ -21,12 +21,7 @@
 
 #include "ui/screen/screen_setup.h"
 
-class CSettings;
-
-namespace Gfx
-{
-class CCamera;
-}
+class CInput;
 
 namespace Ui
 {
@@ -41,9 +36,16 @@ public:
     bool EventProcess(const Event &event) override;
 
 protected:
+    void ChangeSetupButtons();
     void UpdateSetupButtons();
-    void UpdateKey();
+
     void ChangeKey(EventType event);
+    void UpdateKey();
+
+    void ToggleJoystickInvert(EventType type);
+
+protected:
+    CInput* m_input;
 };
 
 } // namespace Ui
