@@ -1,6 +1,6 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * Copyright (C) 2001-2016, Daniel Roux, EPSITEC SA & TerranovaTeam
  * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
@@ -20,26 +20,24 @@
 
 #include "ui/studio.h"
 
-#include "CBot/CBotDll.h"
-
 #include "app/app.h"
 #include "app/pausemanager.h"
 
 #include "common/event.h"
 #include "common/logger.h"
-#include "common/misc.h"
 #include "common/settings.h"
 
 #include "common/resources/resourcemanager.h"
 
-#include "level/robotmain.h"
-#include "level/player_profile.h"
-
 #include "graphics/engine/camera.h"
 #include "graphics/engine/engine.h"
 
+#include "level/player_profile.h"
+#include "level/robotmain.h"
+
 #include "object/object.h"
 
+#include "object/interface/program_storage_object.h"
 #include "object/interface/programmable_object.h"
 
 #include "script/cbottoken.h"
@@ -59,10 +57,12 @@
 #include "ui/controls/list.h"
 #include "ui/controls/map.h"
 #include "ui/controls/shortcut.h"
+#include "ui/controls/slider.h"
 #include "ui/controls/target.h"
 #include "ui/controls/window.h"
 
 #include <stdio.h>
+#include <ctime>
 
 
 namespace Ui
@@ -80,7 +80,7 @@ CStudio::CStudio()
     m_main      = CRobotMain::GetInstancePointer();
     m_interface = m_main->GetInterface();
     m_camera    = m_main->GetCamera();
-    m_pause     = m_engine->GetPauseManager();
+    m_pause     = m_main->GetPauseManager();
     m_settings  = CSettings::GetInstancePointer();
 
     m_program = nullptr;
@@ -354,17 +354,17 @@ bool CStudio::EventFrame(const Event &event)
 
     if (m_script->IsRunning() && (!m_script->GetStepMode() || m_script->IsContinue()))
     {
-        if (m_editorPause != nullptr)
+        if (m_runningPause != nullptr)
         {
-            m_pause->DeactivatePause(m_editorPause);
-            m_editorPause = nullptr;
+            m_pause->DeactivatePause(m_runningPause);
+            m_runningPause = nullptr;
         }
     }
     else
     {
-        if (m_editorPause == nullptr)
+        if (m_runningPause == nullptr)
         {
-            m_editorPause = m_pause->ActivatePause(PAUSE_EDITOR);
+            m_runningPause = m_pause->ActivatePause(PAUSE_ENGINE|PAUSE_MUTE_SOUND);
         }
     }
 
@@ -425,15 +425,9 @@ bool CStudio::EventFrame(const Event &event)
 
 // Indicates whether a character is part of a word.
 
-bool IsToken(int character)
+bool IsToken(char c)
 {
-    char    c;
-
-    c = tolower(GetNoAccent(character));
-
-    return ( (c >= 'a' && c <= 'z') ||
-             (c >= '0' && c <= '9') ||
-             c == '_' );
+    return ( isalnum(c) || c == '_' );
 }
 
 // Seeks if the cursor is on a keyword.
@@ -576,13 +570,14 @@ void CStudio::StartEditScript(CScript *script, std::string name, Program* progra
     m_main->SetSpeed(1.0f);
     m_editCamera = m_camera->GetType();
     m_camera->SetType(Gfx::CAM_TYPE_EDIT);
+    m_editorPause = m_pause->ActivatePause(PAUSE_HIDE_SHORTCUTS, PAUSE_MUSIC_EDITOR);
 
     m_bRunning = m_script->IsRunning();
     m_bRealTime = m_bRunning;
     m_script->SetStepMode(!m_bRealTime);
 
     pw = static_cast<CWindow*>(m_interface->SearchControl(EVENT_WINDOW6));
-    if (pw != nullptr) pw->ClearState(STATE_VISIBLE);
+    if (pw != nullptr) pw->ClearState(STATE_VISIBLE | STATE_ENABLE);
 
     pos = m_editFinalPos = m_editActualPos = m_settings->GetWindowPos();
     dim = m_editFinalDim = m_editActualDim = m_settings->GetWindowDim();
@@ -908,11 +903,12 @@ bool CStudio::StopEditScript(bool bCancel)
     m_interface->DeleteControl(EVENT_WINDOW3);
 
     pw = static_cast<CWindow*>(m_interface->SearchControl(EVENT_WINDOW6));
-    if (pw != nullptr) pw->SetState(STATE_VISIBLE);
+    if (pw != nullptr) pw->SetState(STATE_VISIBLE | STATE_ENABLE);
 
     m_pause->DeactivatePause(m_editorPause);
     m_editorPause = nullptr;
-    m_sound->MuteAll(false);
+    m_pause->DeactivatePause(m_runningPause);
+    m_runningPause = nullptr;
     m_main->SetEditLock(false, true);
     m_camera->SetType(m_editCamera);
     return true;
@@ -982,22 +978,22 @@ void CStudio::UpdateFlux()
     {
         if ( m_bRealTime )  // run?
         {
-            m_pause->DeactivatePause(m_editorPause);
-            m_editorPause = nullptr;
+            m_pause->DeactivatePause(m_runningPause);
+            m_runningPause = nullptr;
         }
         else    // step by step?
         {
-            if (m_editorPause == nullptr)
+            if (m_runningPause == nullptr)
             {
-                m_editorPause = m_pause->ActivatePause(PAUSE_EDITOR);
+                m_runningPause = m_pause->ActivatePause(PAUSE_ENGINE|PAUSE_MUTE_SOUND);
             }
         }
     }
     else    // stop?
     {
-        if (m_editorPause == nullptr)
+        if (m_runningPause == nullptr)
         {
-            m_editorPause = m_pause->ActivatePause(PAUSE_EDITOR);
+            m_runningPause = m_pause->ActivatePause(PAUSE_ENGINE|PAUSE_MUTE_SOUND);
         }
     }
 }
@@ -1032,17 +1028,17 @@ void CStudio::UpdateButtons()
 
     button = static_cast< CButton* >(pw->SearchControl(EVENT_STUDIO_CLONE));
     if ( button == nullptr )  return;
-    button->SetState(STATE_ENABLE, m_program->runnable && !m_bRunning);
+    button->SetState(STATE_ENABLE, m_main->CanPlayerInteract() && (m_program->runnable && !m_bRunning));
 
 
     button = static_cast< CButton* >(pw->SearchControl(EVENT_STUDIO_COMPILE));
     if ( button == nullptr )  return;
-    button->SetState(STATE_ENABLE, m_program->runnable && !m_bRunning);
+    button->SetState(STATE_ENABLE, m_main->CanPlayerInteract() && (m_program->runnable && !m_bRunning));
 
     button = static_cast< CButton* >(pw->SearchControl(EVENT_STUDIO_RUN));
     if ( button == nullptr )  return;
     button->SetIcon(m_bRunning?8:21);  // stop/run
-    button->SetState(STATE_ENABLE, m_program->runnable || m_bRunning);
+    button->SetState(STATE_ENABLE, m_main->CanPlayerInteract() && (m_program->runnable || m_bRunning));
 
     button = static_cast< CButton* >(pw->SearchControl(EVENT_STUDIO_REALTIME));
     if ( button == nullptr )  return;
@@ -1280,9 +1276,9 @@ void CStudio::AdjustDialog()
         {
             pli->SetPos(ppos);
             pli->SetDim(ddim);
-            pli->SetTabs(0, ddim.x-(50.0f+130.0f+16.0f)/640.0f);
+            pli->SetTabs(0, ddim.x-(50.0f+140.0f+16.0f)/640.0f);
             pli->SetTabs(1,  50.0f/640.0f, Gfx::TEXT_ALIGN_RIGHT);
-            pli->SetTabs(2, 130.0f/640.0f);
+            pli->SetTabs(2, 140.0f/640.0f);
 //?         pli->ShowSelect();
         }
 
@@ -1573,7 +1569,7 @@ void CStudio::UpdateDialogList()
     CWindow*        pw;
     CList*          pl;
     int             i = 0;
-    char            time[100];
+    char            timestr[100];
 
     pw = static_cast< CWindow* >(m_interface->SearchControl(EVENT_WINDOW9));
     if ( pw == nullptr )  return;
@@ -1588,8 +1584,9 @@ void CStudio::UpdateDialogList()
     for (auto& prog : programs)
     {
         std::ostringstream temp;
-        TimeToAscii(CResourceManager::GetLastModificationTime(SearchDirectory(false) + prog), time);
-        temp << prog << '\t' << CResourceManager::GetFileSize(SearchDirectory(false) + prog) << "  \t" << time;
+        time_t now = CResourceManager::GetLastModificationTime(SearchDirectory(false) + prog);
+        strftime(timestr, 99, "%x %X", localtime(&now));
+        temp << prog << '\t' << CResourceManager::GetFileSize(SearchDirectory(false) + prog) << "  \t" << timestr;
         pl->SetItemName(i++, temp.str().c_str());
     }
 }
