@@ -1,6 +1,6 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * Copyright (C) 2001-2016, Daniel Roux, EPSITEC SA & TerranovaTeam
  * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
@@ -19,8 +19,6 @@
 
 #pragma once
 
-#include "common/event.h"
-
 #include "graphics/engine/camera.h"
 
 #include <string>
@@ -32,6 +30,7 @@ class CSettings;
 struct Program;
 class CPauseManager;
 struct ActivePause;
+struct Event;
 
 namespace Ui
 {
@@ -115,6 +114,7 @@ protected:
     bool         m_bRunning;
     bool         m_bRealTime;
     ActivePause* m_editorPause = nullptr;
+    ActivePause* m_runningPause = nullptr;
     std::string  m_helpFilename;
 
     StudioDialog m_dialog;
