@@ -1,6 +1,6 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * Copyright (C) 2001-2016, Daniel Roux, EPSITEC SA & TerranovaTeam
  * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
@@ -22,7 +22,7 @@
 
 #include <gtest/gtest.h>
 
-class SystemUtilsLinuxUT : public testing::Test
+class CSystemUtilsLinuxUT : public testing::Test
 {
 protected:
     static const long long SEC = 1000000000;
@@ -31,7 +31,7 @@ protected:
 };
 
 
-TEST_F(SystemUtilsLinuxUT, TimeStampDiff)
+TEST_F(CSystemUtilsLinuxUT, TimeStampDiff)
 {
     SystemTimeStamp before, after;
 
