@@ -1,6 +1,6 @@
 /*
  * This file is part of the Colobot: Gold Edition source code
- * Copyright (C) 2001-2015, Daniel Roux, EPSITEC SA & TerranovaTeam
+ * Copyright (C) 2001-2016, Daniel Roux, EPSITEC SA & TerranovaTeam
  * http://epsitec.ch; http://colobot.info; http://github.com/colobot
  *
  * This program is free software: you can redistribute it and/or modify
@@ -31,7 +31,7 @@ public:
     }
 };
 
-class SystemUtilsWindowsUT : public testing::Test
+class CSystemUtilsWindowsUT : public testing::Test
 {
 protected:
     static const long long SEC = 1000000000;
@@ -39,7 +39,7 @@ protected:
     CSystemUtilsWindowsWrapper m_systemUtils;
 };
 
-TEST_F(SystemUtilsWindowsUT, TimeStampDiff)
+TEST_F(CSystemUtilsWindowsUT, TimeStampDiff)
 {
     m_systemUtils.SetFrequency(SEC);
 
